#!/usr/bin/perl
#
# Usage bin2mif.pl <depth> <width> [<fill>] < input.bin > output.mif
#

eval { use bytes; binmode(STDIN); };

$depth = $ARGV[0] + 0;
$width = $ARGV[1] + 0;
$fill  = $ARGV[2];
$fill  = oct($fill) if ( $fill =~ /^0/ );
$fill  = defined($fill) ? ($fill+0) : 0xff;

if ( $width > 64 ) {
    print STDERR "$0: width > 64 not supported\n";
    exit(1);
} elsif ( $width > 32 ) {
    $bytesper = 8;
} elsif ( $width > 16 ) {
    $bytesper = 4;
} elsif ( $width > 8 ) {
    $bytesper = 2;
} else {
    $bytesper = 1;
}

print "DEPTH = $depth;\n";
print "WIDTH = $width;\n";
print "ADDRESS_RADIX = HEX;\n";
print "DATA_RADIX    = HEX;\n";
print "CONTENT\n";
print "BEGIN\n";

$a = 0;  $d = 0;  $s = 0;  $n = 0;
while ( ($len = read(STDIN, $bytes, 512)) > 0 ) {
    $i = 0;
    while ( $i < $len ) {
	while ( $n < $bytesper ) {
	    # This assumes littleendian data...
	    $d += ord(substr($bytes,$i++,1)) << $s;
	    $s += 8;
	    $n++;
	}
        if ( $n >= $bytesper ) {
	    printf "%X : %X;\n", $a++, $d;
	    $d = 0;  $s = 0;  $n = 0;
	}
    }
}

if ( $a < $depth ) {
    if ( $a == $depth-1 ) {
	printf "%X : %X;\n", $a, $fill;
    } else {
	printf "[%X..%X] : %X;\n", $a, $depth-1, $fill;
    }
}

print "END;\n";

if ( $a > $depth ) {
    print STDERR "$0: Data overflow!\n";
}
